/*!
  * Licensed Materials - Property of IBM
  * 5725N95, 5725P55, 5725N96
  * (C) Copyright IBM Corp. 1999, 2014.
  * US Government Users Restricted Rights- Use, duplication or disclosure
  * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

 /* $Id: viewer.js,v 1.1.2.2 2015/08/24 06:33:41 rakollep Exp $ */
 
TreeController = 
{
	state : false,
	enabled : true,
	leafTypes : [11,6,7,13],
	inlineIFrame : [2,3,4,5],
	cfg : {
		autoHideOnNavigation : true,
		autoHideOnNavigationDelay : 500,		
		maxHeight: 600
	},
	count: 1,
	init : function()
	{
		var vc = ViewerController;
		vc.events.navigate.subscribe(this.handleNavigation,this,true);
		vc.events.load.subscribe(function(type, args, tree) { if (args[0].type == 6) { tree.enabled = false; }}, this, true);
		vc.events.score.subscribe(function(type, args, tree) { if (args[0].type == 6) { tree.enabled = true; }}, this, true);
		var root = vc.root;
		var t = this.tree = new YAHOO.widget.TreeView("tree_container");
		
		var leaf = this.isLeaf(root);
		if (leaf)
		{
			this.addTreeNode(root,t.getRoot(),!leaf);
		}
		else
		{
			this.addTreeNodeChildren(root,t.getRoot(),true);
		}

		t.subscribe("clickEvent",TreeController.nodeClicked,TreeController,true);
		t.subscribe("expandComplete", function(node) {this.markAccessedObj(node, true);	}, TreeController,true);
		t.render();
		this.markAccessedObj(t.root, false);
		var $ = jQuery;
		$("#btn_toc").bind('click', $.proxy(TreeController.toggle, TreeController));
		if (vc.current != null)
		{
			this.handleNavigation("navigation",[vc.current]);
		}
		$("#object-title").html(root.label);
	},
	markAccessedObj: function(node, recursive) {
		var n, nodes = node.children;
		for(var i in nodes) {
			n = nodes[i];
			if(this.accessed(n.data.obj)) {
				if(recursive) {
					if(n.children.length > 0) {
						jQuery(n.getEl().firstChild).find("td.ygtvcontent").addClass("nodeAccessed");
						this.markAccessedObj(n, true);
					} else {
						jQuery(n.getEl()).find("td.ygtvcontent").addClass("nodeAccessed");
					}
				} else {
					jQuery(n.getEl()).find("td.ygtvcontent").addClass("nodeAccessed");
				}
			}
		}
	},
	accessed: function(obj) {
		var result = (obj.lastAccess != null);
		if(result == false) {
			for(var c in obj.children) {
				result = this.accessed(obj.children[c]);
				if(result) {
					return result;
				}
			}
		}
		return result;
	},
	isLeaf : function(obj)
	{
		return (jQuery.inArray(obj.type,this.leafTypes) != -1);
	},
	addTreeNode : function(obj, parentNode, addChildren)
	{
		var n = new YAHOO.widget.TextNode({ label: obj.label, obj: obj, expanded:false, hasIcon: false }, parentNode);
		if (addChildren && !this.isLeaf(obj))
		{
			this.addTreeNodeChildren(obj,n,true);
		}
	},
	addTreeNodeChildren : function(obj, parentNode, addChildren)
	{
		if (obj.children)
		{
			var c;
			for (var i=0; i<obj.children.length; i++)
			{
				c = obj.children[i];
				if (!ViewerController.isRemoved(c))
				{
					this.addTreeNode(obj.children[i], parentNode, true);
				}
			}
		}
	},
	handleNavigation : function(type, args)
	{
		var yd = YAHOO.util.Dom;
		var obj = args[0];
		
		if (this.current != null)
		{
			var last = this.tree.getNodeByProperty("obj",this.current);
			if (last != null)
			{
				yd.removeClass(last.getEl(),"activeNode");
			}
		}
		
		var found = false;
		var p = obj;
		do
		{
			if (this.showNode(p))
			{
				found = true;
				this.current = p;
				break;
			}
		}
		while ((p = p.parent) != null);
		
		if (!found) this.tree.collapseAll();
		var me = this;
		// hide the tree
		if (this.cfg.autoHideOnNavigation) setTimeout(function() {me.toggle(true);} ,this.cfg.autoHideOnNavigationDelay);
	},
	showNode : function(obj)
	{
		var s = false;
		var node = this.tree.getNodeByProperty("obj",obj);
		if (node != null)
		{
			s = true;

			var p = node;
			while ((p = p.parent) != null)
			{
				p.expand();
			}			
			YAHOO.util.Dom.addClass(node.getEl(),"activeNode");
			jQuery(node.getEl()).find("td.ygtvcontent").addClass("nodeAccessed");
		}
		return s;
	},
	nodeClicked : function(e)
	{
		if (this.enabled)
		{
			ViewerController.navigateTo(e.node.data.obj);
		}
		return false;
	},
	toggle : function(hide) {
		
		var $tree = jQuery("#tree"), hidden = ($tree.css('display') == 'none');
		if(hide === hidden) {
			return;
		}
		var contentHeight = jQuery("#content").innerHeight(); 
		var h = (contentHeight > this.cfg.maxHeight) ? this.cfg.maxHeight : contentHeight;
		jQuery("#tree_container").css('height', h - 92);
		var osVersion = getOsVersion();
		if(osVersion >= 8.0)
			jQuery("#tree_container").css('-webkit-overflow-scrolling', 'auto');
		else
			jQuery("#tree_container").css('-webkit-overflow-scrolling', 'touch');		
		var $tocBtn = jQuery("#btn_toc");
		var tocPos = $tocBtn.offset();
		var x = tocPos.left + ($tocBtn.width()/2) - $tree.width()/2 - 7, y = tocPos.top + ($tocBtn.height()/2) + 45;
		$tree.css({top: y, left: x, height: h-70, "z-index": 9999})
		if(hide === true) {
			$tree.slideUp('slow');
		} else if(hide === false) {
			$tree.slideDown('slow');
		} else {
			$tree.slideToggle('slow');
		}
	}
}

CounterController =  {
	init : function() {
		var vc = ViewerController;
		vc.events.navigate.subscribe(this.update,this,true);
		vc.events.load.subscribe(this.update,this,true);
		vc.events.score.subscribe(this.update,this,true);
	},
	update : function()	{
		var p = ViewerController.getPosition();
		jQuery("#counter_t").html((p.position+1)+" of "+p.total);
	}
}

CounterController.init();

ButtonController = 
{
	buttons : {},
	init : function()
	{
		
		this.createButton("prev", function() { ViewerController.previous() }, function() { return !ViewerController.previous(true); });
		this.createButton("next", function() { ViewerController.next() }, function() { return !ViewerController.next(true); });
		this.createButton("glossary", function() { ViewerController.openGlossary(), function(){return false;} });
		this.createButton("help", function() { ViewerController.openHelp(), function(){return false;} });
		var vc = ViewerController;
		vc.events.navigate.subscribe(this.update,this,true);				
		vc.events.load.subscribe(function(type, args, obj) { if (args[0].type == 6) { obj.disableAll(); } }, this, true);
		vc.events.score.subscribe(function(type, args, obj) { if (args[0].type == 6) obj.update(); }, this, true);	
	},
	update : function()
	{
		for (var id in this.buttons)
		{
			var cfg = this.buttons[id];
			cfg.btn.set("disabled", cfg.update());
		}
	},
	disableAll : function()
	{
		for (var id in this.buttons)
		{
			var cfg = this.buttons[id];
			cfg.btn.set("disabled", true);
		}
	},
	createButton : function(id, fnClick, fnUpdate)
	{
		if (YAHOO.util.Dom.get("btn_"+id) != null)
		{
			var btn = new YAHOO.widget.Button("btn_"+id, { name:"id", onclick:{ fn: fnClick } });
			this.buttons[id] = { "btn": btn, "update": fnUpdate };
		}
	}
}

ViewerController.loadingPanel = null;
ViewerController.loadedScriptRegExps = [];
ViewerController.cfg = {
		devMode: false,
		externalContentViewerServlet: "ExternalContentViewer",
		maxLevel: 5 //max level to go up from the target evt when checking if the user is really dragging items.
};
//overwrite the handler.
ViewerController.handleNavRequest = function(type, args){
	var r = false;	
	var type = args[0][1];
	// handle inline navigation links
	if (type == "0") {
		var id = args[0][0];
		var obj = vc.engine.getObject(id);
		vc.events.navigate.fire(obj[0], vc.engine.getURL(id));
	} else {
		r = true;
	}

	return r;
}

//open glossary
ViewerController.openGlossary = function() 
{
	alert('No Glossary currently available.');
}

// open help
ViewerController.openHelp = function() 
{
	 alert("No Help currently available");
}

ViewerController.filterHTML = function(html) {
	var regExps = vc.loadedScriptRegExps;
	var len = regExps.length, r;
	for(var i = 0; i < len; i ++) {
		r = regExps[i];
		//reset the lastIndex property so that regex can be used many times.
		r.lastIndex = 0;
		if(r.test(html)) {
			html = html.replace(r, "");
		}
	}
	return html;
}

ViewerController.events.group_loaded = new YAHOO.util.CustomEvent("group_loaded");

ViewerController.events.before_navigate.subscribe(function(type,args, me) {
	//warning the user if he navigates to another page when in assessment mode.
	if(vc.assessmentStart) {
		if(confirm (AssessmentEngine.closeMsg)) {
			vc.assessmentStart = false;
			AssessmentEngine.finalize({type : "unload"});
		}
		return false;
	}
	return true;
});

ViewerController.events.navigate.subscribe(function(type,args) {
	var $ = jQuery;
	var ye = YAHOO.util.Event;
	ye.removeListener(window, "load");
	var currentURL = args[1];
	if(currentURL.indexOf(vc.cfg.externalContentViewerServlet)>=0 || (args[0] && $.inArray(args[0].type, [2,3,4,5])>=0 )) {
		vc.toggleAssessmentBtn(false);
		$("#content").empty().append("<iframe src='"+ currentURL+"' style ='width: 100%; height: 100%'></iframe>");
	} else {
		$.ajax({
			type: "GET",
			url: currentURL,
			dataType: "html",
			beforeSend: function() {
				vc.loadingPanel.show();
			},
			complete: function(){
				vc.loadingPanel.hide();
				var matches = /__scrollTop=(\d+)/.exec(currentURL),
					scrollTop = matches ? matches[1] : 0;
				// jump to the scroll top position if __scrollTop parameter exists
				$("#content").scrollTop(scrollTop);
			},
			success: function(html) {
				var i,c = vc.current.children, fn;
				html = vc.filterHTML(html);
				//unbind events before removing them.
				$("body>*:not('#layout-doc,#tree,#loading_c,#loading_mask,#assessPageContainer')").each(function() {
					ye.purgeElement(this, true);
				}).remove();
				var $grouping = $("#content"); 
				ye.purgeElement($grouping[0], true);
				vc.events.media_unloaded.fire();
				$(window).unbind("load").unbind('.elements');
				$grouping.html(html);
				//manually initialize the engine
				var loadEvtHandlers = YAHOO.util.Event.getListeners(window, "load"), handler;
				if(loadEvtHandlers) {
					for ( i = 0; i < loadEvtHandlers.length; i++) {
						handler = loadEvtHandlers[i];
						if(handler.obj) {
							if(handler.adjust === true) {
								handler.fn.call(handler.obj);
							} else if(typeof(handler.adjust) == "object") {
								handler.fn.call(handler.adjust);
							}
						}
					}
				}
				$grouping.find('div.content_container').waitForImages(function() {
					fn = window["f_"+this.id.substring(1)+"_load"]; 
					if(fn) {
						fn();
					}
				});
				$grouping.waitForImages(function() {
					//trigger layout animation
					var g = $grouping.find("div.group_container");
					if(g.length == 1) {
						fn = window["f_" + g[0].id];
						if(fn) {
							fn();
						}
					}
					vc.events.group_loaded.fire($grouping);
				});
				// DeliveryReview may not be available in some pages such as assessment report, assessment confirmation...
				if(typeof(DeliveryReview) !== "undefined") {
					DeliveryReview.events.comment_added.subscribe(function(type, args, me){
						//reload **the content** after a comment is added.
						currentURL = DeliveryReview.appendScrollTopParam(currentURL, $("#content").scrollTop());
						vc.events.navigate.fire(me, currentURL);
						return false;
					});
				}
				//show/hide assessment buttons
				if(vc.assessmentStart || vc.cfg.devMode) {
					vc.toggleAssessmentBtn(true);
				} else {
					vc.toggleAssessmentBtn(false);
				}
			},
			error: function(d) {
				$("#content").html("<div id='request_error'><p>Errors occured loading the resource</p></div>")
			}
		});
	}
});

ViewerController.toggleAssessmentBtn = function(show){
	if(show) {
		jQuery("#assess_btn").show();
		jQuery("#assess_btn_bg").show();
		jQuery("#tmr").show();
	} else {
		jQuery("#assess_btn").hide();
		jQuery("#assess_btn_bg").hide();
		jQuery("#tmr").hide();
	}
}

ViewerController.loadingInit = function () {
	//cache script files included in the result.
	jQuery.ajaxSetup({
		cache: true
	});
	vc.loadedScriptRegExps = jQuery("head>script[src!='']").map(function(i, el) {
		var s = this.src;
		var src = s.substring(s.lastIndexOf("/") + 1);
		if(src != "" && src != "delivery_core.js") {
			return new RegExp("<script[^>]*src=\"[^>]*?/"+ src +"\"[^>]*>(\\s*?)</script\>", "gim");
		}
		return null;
	}).get();
	var cfg = { 
		width: "240px", 
	    fixedcenter: true, 
	    close: false, 
	    draggable: false, 
	    modal: true,
	    visible: false,
	    zIndex:1000, 
		iframe: false
	};
	var p = vc.loadingPanel =  new YAHOO.widget.Panel("loading",  cfg);
	p.setHeader("Loading, please wait...");
	p.setBody("<img src=\""+vc.config.loadingImage+"\"/>");
	p.render(document.body);
}

ViewerController.swipeSupport = function(){
	
	jQuery("#content").bind("swipeleft swiperight", function(evt){//debugger;
	
		var swipeLeft = (evt.type === "swipeleft");
		
		var curNode = evt.target, levelCnt = 0, maxLevel = vc.cfg.maxLevel, dragging = false;
		//check if we're dragging items and that can trigger event 'swipe'
		while(levelCnt < maxLevel && !dragging) {
			//check if current node has class ui-draggable or ui-sortable or draggable-item
			dragging = /(^|\s)(ui\-draggable|ui\-sortable|draggable\-item)(\s|$)/.test(curNode.className);
			levelCnt ++;
			curNode = curNode.parentNode;
		}
		if(!dragging) {
			if(vc.assessmentStart) {
				var btnId = swipeLeft ? "asses_btn_next" : "assess_btn_prev";
				jQuery("#" + btnId).trigger("click");
			} else {
				var method = swipeLeft ? "next" : "previous";				
				vc[method]();
			}
		}
	});
	
	jQuery.event.special.swipe.horizontalDistanceThreshold = 300;
}

jQuery(document).ready(function()
{
	vc.loadingInit();
	TreeController.init();	
	ButtonController.init();	

	var layout = new YAHOO.widget.Layout({
		units: [
			{ position: 'top', height: 120, body: 'header' },
			{ position: 'bottom', height: 50, body: 'footer' },
			{ position: 'center', body: 'content' },
			{ position: 'left', width: 38 ,body: 'left' },
			{ position: 'right', width: 31, body: 'right' }
		]
	});
	layout.render();
	vc.swipeSupport();
});

function getOsVersion() {
    var agent = window.navigator.userAgent,
        start = agent.indexOf( 'OS ' );

    if ( /iphone|ipod|ipad/.test( agent.toLowerCase() ) && start > -1 ) {
        return window.Number( agent.substr( start + 3, 3 ).replace( '_', '.' ) );
    } else {
        return 0;
    };

};